@@ -23,25 +23,16 @@
 #include <QFileInfo>
 #include "Game.h"
 
-Game::Game(const QString& weights, const QString& opt, const QString& binary,
-           const QStringList& commands) :
+Game::Game(const Engine& engine) :
     QProcess(),
-    m_cmdLine(""),
-    m_binary(binary),
-    m_commands(commands),
+    m_engine(engine),
+    m_isHandicap(false),
     m_resignation(false),
     m_blackToMove(true),
     m_blackResigned(false),
     m_passes(0),
     m_moveNum(0)
 {
-#ifdef WIN32
-    m_binary.append(".exe");
-#endif
-    if (!QFileInfo::exists(m_binary)) {
-        m_binary.remove(0, 2); // ./leelaz -> leelaz
-    }
-    m_cmdLine = m_binary + " " + opt + " " + weights;
     m_fileName = QUuid::createUuid().toRfc4122().toHex();
 }
 
@@ -170,8 +161,10 @@ void Game::checkVersion(const VersionTuple &min_version) {
     }
 }
 
-bool Game::gameStart(const VersionTuple &min_version) {
-    start(m_cmdLine);
+bool Game::gameStart(const VersionTuple &min_version,
+                     const QString &sgf,
+                     const int moves) {
+    start(m_engine.getCmdLine());
     if (!waitForStarted()) {
         error(Game::NO_LEELAZ);
         return false;
@@ -180,7 +173,44 @@ bool Game::gameStart(const VersionTuple &min_version) {
     // check any return values.
     checkVersion(min_version);
     QTextStream(stdout) << "Engine has started." << endl;
-    for (auto command : m_commands) {
+    //If there is an sgf file to start playing from then it will contain
+    //whether there is handicap in use. If there is no sgf file then instead,
+    //check whether there are any handicap commands to send (these fail
+    //if the board is not empty).
+    //Then send the rest of the GTP commands after any SGF has been loaded so
+    //that they can override any settings loaded from the SGF.
+    if (!sgf.isEmpty()) {
+        QFile sgfFile(sgf + ".sgf");
+        if (!sgfFile.exists()) {
+            QTextStream(stdout) << "Cannot find sgf file " << sgf << endl;
+            exit(EXIT_FAILURE);
+        }
+        sgfFile.open(QIODevice::Text | QIODevice::ReadOnly);
+        const auto sgfData = QTextStream(&sgfFile).readAll();
+        const auto re = QRegularExpression("HA\\[\\d+\\]");
+        const auto match = re.match(sgfData);
+        m_isHandicap = match.hasMatch();
+        sgfFile.close();
+        if (moves == 0) {
+            loadSgf(sgf);
+        } else {
+            loadSgf(sgf, moves);
+        }
+        setMovesCount(moves);
+    } else {
+        for (auto command : m_engine.m_commands.filter("handicap")) {
+            QTextStream(stdout) << command << endl;
+            if (!sendGtpCommand(command))
+            {
+                QTextStream(stdout) << "GTP failed on: " << command << endl;
+                exit(EXIT_FAILURE);
+            }
+            m_isHandicap = true;
+            m_blackToMove = false;
+        }
+    }
+    const auto re = QRegularExpression("^((?!handicap).)*$");
+    for (auto command : m_engine.m_commands.filter(re)) {
         QTextStream(stdout) << command << endl;
         if (!sendGtpCommand(command))
         {
@@ -188,7 +218,7 @@ bool Game::gameStart(const VersionTuple &min_version) {
             exit(EXIT_FAILURE);
         }
     }
-    QTextStream(stdout) << "Thinking time set." << endl;
+    QTextStream(stdout) << "Starting GTP commands sent." << endl;
     return true;
 }
 
@@ -206,7 +236,10 @@ void Game::move() {
 
 void Game::setMovesCount(int moves) {
     m_moveNum = moves;
-    m_blackToMove = (moves % 2) == 0;
+    //The game always starts at move 0 (GTP states that handicap stones are not part
+    //of the move history), so if there is no handicap then black moves on even
+    //numbered turns but if there is handicap then black moves on odd numbered turns.
+    m_blackToMove = (moves % 2) == (m_isHandicap ? 1 : 0);
 }
 
 bool Game::waitReady() {
@@ -354,13 +387,13 @@ bool Game::loadSgf(const QString &fileName) {
     return sendGtpCommand(qPrintable("loadsgf " + fileName + ".sgf"));
 }
 
-bool Game::fixSgf(QString& weightFile, bool resignation) {
-    QFile sgfFile(m_fileName + ".sgf");
-    if (!sgfFile.open(QIODevice::Text | QIODevice::ReadOnly)) {
-        return false;
-    }
-    QString sgfData = sgfFile.readAll();
-    QRegularExpression re("PW\\[Human\\]");
+bool Game::loadSgf(const QString &fileName, const int moves) {
+    QTextStream(stdout) << "Loading " << fileName + ".sgf with " << moves << " moves" << endl;
+    return sendGtpCommand(qPrintable("loadsgf " + fileName + ".sgf " + QString::number(moves + 1)));
+}
+
+void Game::fixSgfPlayer(QString& sgfData, const Engine& whiteEngine) {
+    QRegularExpression oldPlayer("PW\\[Human\\]");
     QString playerName("PB[Leela Zero ");
     QRegularExpression le("PB\\[Leela Zero \\S+ ");
     QRegularExpressionMatch match = le.match(sgfData);
@@ -368,10 +401,36 @@ bool Game::fixSgf(QString& weightFile, bool resignation) {
         playerName = match.captured(0);
     }
     playerName = "PW" + playerName.remove(0, 2);
-    playerName += weightFile.left(8);
+    playerName += whiteEngine.getNetworkFile().left(8);
     playerName += "]";
-    sgfData.replace(re, playerName);
+    sgfData.replace(oldPlayer, playerName);
+}
 
+void Game::fixSgfComment(QString& sgfData, const Engine& whiteEngine,
+    const bool isSelfPlay) {
+    QRegularExpression oldComment("(C\\[Leela Zero)( options:.*)\\]");
+    QString comment("\\1");
+    if (!isSelfPlay) {
+        comment += " Black";
+    }
+    comment += "\\2 Starting GTP commands:";
+    for (const auto command : m_engine.m_commands) {
+        comment += " " + command;
+    }
+    if (!isSelfPlay) {
+        comment += " White options:";
+        comment += whiteEngine.m_options + " " + whiteEngine.m_network;
+        comment += " Starting GTP commands:";
+        for (const auto command : whiteEngine.m_commands) {
+            comment += " " + command;
+        }
+    }
+    comment += "]";
+    comment.replace(QRegularExpression("\\s\\s+"), " ");
+    sgfData.replace(oldComment, comment);
+}
+
+void Game::fixSgfResult(QString& sgfData, const bool resignation) {
     if (resignation) {
         QRegularExpression oldResult("RE\\[B\\+.*\\]");
         QString newResult("RE[B+Resign] ");
@@ -384,7 +443,18 @@ bool Game::fixSgf(QString& weightFile, bool resignation) {
         QString noPass(")");
         sgfData.replace(lastpass, noPass);
     }
+}
 
+bool Game::fixSgf(const Engine& whiteEngine, const bool resignation,
+    const bool isSelfPlay) {
+    QFile sgfFile(m_fileName + ".sgf");
+    if (!sgfFile.open(QIODevice::Text | QIODevice::ReadOnly)) {
+        return false;
+    }
+    QString sgfData = sgfFile.readAll();
+    fixSgfPlayer(sgfData, whiteEngine);
+    fixSgfComment(sgfData, whiteEngine, isSelfPlay);
+    fixSgfResult(sgfData, resignation);
     sgfFile.close();
     if (sgfFile.open(QFile::WriteOnly | QFile::Truncate)) {
         QTextStream out(&sgfFile);
