@@ -1,6 +1,6 @@
 /*
     This file is part of Leela Zero.
-    Copyright (C) 2017-2018 Gian-Carlo Pascutto and contributors
+    Copyright (C) 2017-2019 Gian-Carlo Pascutto and contributors
 
     Leela Zero is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,6 +14,17 @@
 
     You should have received a copy of the GNU General Public License
     along with Leela Zero.  If not, see <http://www.gnu.org/licenses/>.
+
+    Additional permission under GNU GPL version 3 section 7
+
+    If you modify this Program, or any covered work, by linking or
+    combining it with NVIDIA Corporation's libraries from the
+    NVIDIA CUDA Toolkit and/or the NVIDIA CUDA Deep Neural
+    Network library and/or the NVIDIA TensorRT inference library
+    (or a modified version of those libraries), containing parts covered
+    by the terms of the respective license agreement, the licensors of
+    this Program grant you additional permission to convey the resulting
+    work.
 */
 
 #include "GameState.h"
@@ -148,6 +159,10 @@ const TimeControl& GameState::get_timecontrol() const {
     return m_timecontrol;
 }
 
+void GameState::set_timecontrol(const TimeControl& timecontrol) {
+    m_timecontrol = timecontrol;
+}
+
 void GameState::set_timecontrol(int maintime, int byotime,
                                 int byostones, int byoperiods) {
     TimeControl timecontrol(maintime, byotime,
@@ -301,3 +316,7 @@ const FullBoard& GameState::get_past_board(int moves_ago) const {
     assert(m_movenum + 1 <= game_history.size());
     return game_history[m_movenum - moves_ago]->board;
 }
+
+const std::vector<std::shared_ptr<const KoState>>& GameState::get_game_history() const {
+    return game_history;
+}
