@@ -1,6 +1,6 @@
 /*
     This file is part of Leela Zero.
-    Copyright (C) 2017-2018 Gian-Carlo Pascutto
+    Copyright (C) 2017-2019 Gian-Carlo Pascutto
 
     Leela Zero is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,6 +14,17 @@
 
     You should have received a copy of the GNU General Public License
     along with Leela Zero.  If not, see <http://www.gnu.org/licenses/>.
+
+    Additional permission under GNU GPL version 3 section 7
+
+    If you modify this Program, or any covered work, by linking or
+    combining it with NVIDIA Corporation's libraries from the
+    NVIDIA CUDA Toolkit and/or the NVIDIA CUDA Deep Neural
+    Network library and/or the NVIDIA TensorRT inference library
+    (or a modified version of those libraries), containing parts covered
+    by the terms of the respective license agreement, the licensors of
+    this Program grant you additional permission to convey the resulting
+    work.
 */
 
 #ifndef UCTSEARCH_H_INCLUDED
@@ -78,10 +89,10 @@ public:
 
     /*
         Default memory limit in bytes.
-        ~1.3GiB on 32-bits and about 5.2GiB on 64-bits.
+        ~1.6GiB on 32-bits and about 5.2GiB on 64-bits.
     */
     static constexpr size_t DEFAULT_MAX_MEMORY =
-        (sizeof(void*) == 4 ? 1'325'000'000 : 5'200'000'000);
+        (sizeof(void*) == 4 ? 1'600'000'000 : 5'200'000'000);
 
     /*
         Minimum allowed size for maximum tree size.
@@ -103,6 +114,7 @@ public:
     void ponder();
     bool is_running() const;
     void increment_playouts();
+    std::string explain_last_think() const;
     SearchResult play_simulation(GameState& currstate, UCTNode* const node);
 
 private:
@@ -110,11 +122,11 @@ private:
     void dump_stats(FastState& state, UCTNode& parent);
     void tree_stats(const UCTNode& node);
     std::string get_pv(FastState& state, UCTNode& parent);
-    void dump_analysis(int playouts);
+    std::string get_analysis(int playouts);
     bool should_resign(passflag_t passflag, float besteval);
     bool have_alternate_moves(int elapsed_centis, int time_for_move);
     int est_playouts_left(int elapsed_centis, int time_for_move) const;
-    size_t prune_noncontenders(int elapsed_centis = 0, int time_for_move = 0,
+    size_t prune_noncontenders(int color, int elapsed_centis = 0, int time_for_move = 0,
                                bool prune = true);
     bool stop_thinking(int elapsed_centis = 0, int time_for_move = 0) const;
     int get_best_move(passflag_t passflag);
@@ -130,6 +142,7 @@ private:
     std::atomic<bool> m_run{false};
     int m_maxplayouts;
     int m_maxvisits;
+    std::string m_think_output;
 
     std::list<Utils::ThreadGroup> m_delete_futures;
 
