@@ -62,11 +62,11 @@ int main(int argc, char *argv[]) {
             "lower:upper", "0.0:35.0");
     QCommandLineOption gamesNumOption(
         {"g", "gamesNum"},
-            "Play 'gamesNum' games on one GPU at the same time.",
+            "Play 'gamesNum' games on one device (GPU/CPU) at the same time.",
             "num", "1");
     QCommandLineOption gpusOption(
         {"u", "gpus"},
-            "Index of the GPU to use for multiple GPUs support.",
+            "Index of the device(s) to use for multiple devices support.",
             "num");
     QCommandLineOption keepSgfOption(
         {"k", "keepSgf" },
@@ -110,6 +110,8 @@ int main(int argc, char *argv[]) {
     }
 
     QTextStream(stdout) << "validation v" << VALIDATION_VERSION << endl;
+
+    auto const keepPath = parser.value(keepSgfOption);
     if (parser.isSet(keepSgfOption)) {
         if (!QDir().mkpath(parser.value(keepSgfOption))) {
             QTextStream(stdout) << "Couldn't create output directory for self-play SGF files!"
@@ -143,8 +145,7 @@ int main(int argc, char *argv[]) {
 
     Console *cons = nullptr;
     Validation *validate = new Validation(gpusNum, gamesNum, gpusList,
-                                          engines,
-                                          parser.value(keepSgfOption), &mutex,
+                                          engines, keepPath, &mutex,
                                           h0, h1);
     QObject::connect(&app, &QCoreApplication::aboutToQuit, validate, &Validation::storeSprt);
     validate->loadSprt();
